package core

import (
	"git.akyoto.dev/cli/q/src/asm"
	"git.akyoto.dev/cli/q/src/errors"
	"git.akyoto.dev/cli/q/src/expression"
	"git.akyoto.dev/cli/q/src/scope"
	"git.akyoto.dev/cli/q/src/token"
	"git.akyoto.dev/cli/q/src/types"
	"git.akyoto.dev/cli/q/src/x86"
)

// CompileAssignDivision compiles an assign statement that has quotient and remainder on the left side and division on the right.
func (f *Function) CompileAssignDivision(expr *expression.Expression) error {
	var (
		left              = expr.Children[0]
		right             = expr.Children[1]
		quotientVariable  *scope.Variable
		remainderVariable *scope.Variable
		err               error
	)

	if expr.Token.Kind == token.Define {
		quotientVariable, err = f.Define(left.Children[0])

		if err != nil {
			return err
		}

		remainderVariable, err = f.Define(left.Children[1])

		if err != nil {
			return err
		}

		quotientVariable.Type = types.Int
		remainderVariable.Type = types.Int
		f.AddVariable(quotientVariable)
		f.AddVariable(remainderVariable)
	} else {
		quotient := left.Children[0]
		name := quotient.Token.Text(f.File.Bytes)
		quotientVariable = f.VariableByName(name)

		if quotientVariable == nil {
			return errors.New(&errors.UnknownIdentifier{Name: name}, f.File, quotient.Token.Position)
		}

		remainder := left.Children[1]
		name = remainder.Token.Text(f.File.Bytes)
		remainderVariable = f.VariableByName(name)

		if remainderVariable == nil {
			return errors.New(&errors.UnknownIdentifier{Name: name}, f.File, remainder.Token.Position)
		}

		defer f.UseVariable(quotientVariable)
		defer f.UseVariable(remainderVariable)
	}

	dividend := right.Children[0]
	_, dividendRegister, isTemporary, err := f.Evaluate(dividend)

	if err != nil {
		return err
	}

	divisor := right.Children[1]
	err = f.Execute(right.Token, dividendRegister, divisor)
	f.RegisterRegister(asm.MOVE, quotientVariable.Register, x86.RAX)
	f.RegisterRegister(asm.MOVE, remainderVariable.Register, x86.RDX)

	if isTemporary {
		f.FreeRegister(dividendRegister)
	}

	return err
}
