package core

import (
	"git.akyoto.dev/cli/q/src/config"
	"git.akyoto.dev/cli/q/src/expression"
	"git.akyoto.dev/cli/q/src/token"
)

// Fold will try to precalculate the results of operations with constants.
func (f *Function) Fold(expr *expression.Expression) error {
	if !config.ConstantFold {
		return nil
	}

	if expr == nil {
		return nil
	}

	if expr.IsLeaf() {
		if expr.Token.IsNumeric() {
			value, err := f.ToNumber(expr.Token)
			expr.Value = value
			expr.IsFolded = true
			return err
		}

		return nil
	}

	if expr.Token.Kind == token.Period {
		left := expr.Children[0]
		leftText := left.Token.Text(f.File.Bytes)
		right := expr.Children[1]
		rightText := right.Token.Text(f.File.Bytes)
		constant, isConst := f.All.Constants[f.Package+"."+leftText+"."+rightText]

		if !isConst {
			return nil
		}

		value, err := ToNumber(constant.Token, constant.File)
		expr.Value = value
		expr.IsFolded = true
		return err
	}

	canFold := true

	for _, child := range expr.Children {
		f.Fold(child)

		if !child.IsFolded {
			canFold = false
		}
	}

	if !canFold || len(expr.Children) != 2 {
		return nil
	}

	a := expr.Children[0].Value
	b := expr.Children[1].Value

	switch expr.Token.Kind {
	case token.Or:
		expr.Value = a | b
	case token.And:
		expr.Value = a & b
	case token.Xor:
		expr.Value = a ^ b
	case token.Add:
		expr.Value = a + b
	case token.Sub:
		expr.Value = a - b
	case token.Mul:
		expr.Value = a * b
	case token.Div:
		expr.Value = a / b
	case token.Mod:
		expr.Value = a % b
	case token.Shl:
		expr.Value = a << b
	case token.Shr:
		expr.Value = a >> b
	default:
		return nil
	}

	expr.IsFolded = true
	return nil
}
